@@ -130,6 +130,14 @@ class PointWiseDesc_v8 : public BackendDescriptor {
 #if (CUDNN_VERSION >= 8400)
             case CUDNN_POINTWISE_BINARY_SELECT:
                 return 4;
+#endif
+#if (CUDNN_VERSION >= 8900)
+            case CUDNN_POINTWISE_RECIPROCAL:
+                return 2;
+#endif
+#ifndef NO_DEFAULT_IN_SWITCH
+            default:
+                return -1;
 #endif
         }
         return -1;
