@@ -125,7 +125,7 @@ void scale_and_bias_tensor_cpu(
     // Scale and bias per channel basis. Assumes NHWC format.
     for (int i = 0; i < inputSize; i++) {
         int c = i % inputDims[0];
-        outputData[i] = inputData[i] * scaleData[c] + biasData[c];
+        outputData[i] = (scale_bias_type)inputData[i] * scaleData[c] + biasData[c];
     }
 }
 
@@ -160,12 +160,13 @@ void gen_stats_cpu(
     const int64_t inputSize,
     const int64_t* inputDims)
 {
-    std::vector<int64_t> totals((size_t)inputDims[0], 0);
+    int64_t channel_dim = inputDims[1];
+    std::vector<int64_t> totals((size_t)channel_dim, 0);
     for (int i = 0; i < inputSize; i++) {
-        int channel_index = i % inputDims[0];
+        int channel_index = i % channel_dim;
 
         // Sum
-        outputData[channel_index].first = outputData[channel_index].first + inputData[i];
+        outputData[channel_index].first = outputData[channel_index].first + (float)inputData[i];
         totals[channel_index] = totals[channel_index] + 1;
     }
 
@@ -175,11 +176,11 @@ void gen_stats_cpu(
     }
 
     for (int i = 0; i < inputSize; i++) {
-        int channel_index = i % inputDims[0];
+        int channel_index = i % channel_dim;
 
         // Sum of squares
-        T_ELEM diff = (inputData[i] - outputData[channel_index].first) * (inputData[i] - outputData[channel_index].first);
-        outputData[channel_index].second = outputData[channel_index].second + diff;
+        T_ELEM diff = ((float)inputData[i] - outputData[channel_index].first) * ((float)inputData[i] - outputData[channel_index].first);
+        outputData[channel_index].second = (T_ELEM)outputData[channel_index].second + diff;
     }
 
     // Calculate the variance for the channel. Assumes NHWC format.
@@ -196,11 +197,11 @@ void batch_normalize(
     const int64_t inputSize,
     const int64_t* inputDims)
 {   
-
+    int64_t channel_dim = inputDims[1];
     // Loop through each element in the input and normalize it based on what batch it belongs to
     for (int i = 0; i < inputSize; i++) {
-        int batch_index = i % inputDims[0];
-        outputData[i] = (inputData[i] - stats[batch_index].first) / (T_ELEM) std::sqrt(stats[batch_index].second);
+        int batch_index = i % channel_dim;
+        outputData[i] = ((float)inputData[i] - stats[batch_index].first) / (float) std::sqrt(stats[batch_index].second);
     }
 }
 
